import { Pipe, PipeTransform } from '@angular/core';
import { DatePipe, DecimalPipe } from '@angular/common';
import { Globals } from './global';
import { DomSanitizer } from '@angular/platform-browser';
import { orderBy } from 'lodash';

@Pipe({
  name: 'dateFormat'
})
export class dateFormatPipe extends DatePipe implements PipeTransform {
  constructor(private global: Globals) {
    super('en-us');
  }

  transform(value: any, args?: any): any {
    return value && super.transform(value, this.global.DateFormat);
  }
}


@Pipe({
  name: 'dateTimeFormat'
})
export class dateTimeFormatPipe extends DatePipe implements PipeTransform {
  constructor(private global: Globals) {
    super('en-us');
  }

  transform(value: any, args?: any): any {
    return value && super.transform(value, this.global.DateTimeFormat);
  }
}


@Pipe({
  name: 'dateTimeFormat24H'
})
export class dateTimeFormat24HPipe extends DatePipe implements PipeTransform {
  constructor(private global: Globals) {
    super('en-us');
  }
  transform(value: any, args?: any): any {
    return value && super.transform(value, this.global.DateTimeFormat24H);
  }
}
@Pipe({
  name: 'filterBytext'
})
export class FilterBytextPipe implements PipeTransform {

  transform(items: any[], prop: string, searchText: string): any {

    if (!items) return [];
    if (!searchText) return items;
    searchText = searchText.toLocaleLowerCase();
    return items.filter(item => item[prop].toLocaleLowerCase().indexOf(searchText) !== -1);
  }
}

@Pipe({
  name: 'uniquePipe'
})
export class UniquePipe implements PipeTransform {

  transform(value: any, prop: string): any {
    const array = value.map(x => x[prop])
      .filter((v, i, s) => s.indexOf(v) === i)
      .sort();

    return array;
  }
}

@Pipe({
  name: 'safe'
})
export class SafePipe implements PipeTransform {
  constructor(private sanitizer: DomSanitizer) {}

  /**
   * 
   * @param value 
   * @param securityContext 
   *    NONE: 0
   *    HTML: 1
   *    STYLE: 2
   *    SCRIPT: 3
   *    URL: 4
   *    RESOURCE_URL: 5
   * @returns 
   */
  transform(value: any, securityContext: 'HTML' | 'STYLE' | 'SCRIPT' | 'URL' | 'RESOURCE_URL'): any {
    switch(securityContext) {
      case 'HTML': return this.sanitizer.bypassSecurityTrustHtml(value);
      case 'STYLE': return this.sanitizer.bypassSecurityTrustStyle(value);
      case 'SCRIPT': return this.sanitizer.bypassSecurityTrustScript(value);
      case 'URL': return this.sanitizer.bypassSecurityTrustUrl(value);
      case 'RESOURCE_URL': return this.sanitizer.bypassSecurityTrustResourceUrl(value);
      default: return this.sanitizer.sanitize(0, value);
    }
  }
}

@Pipe({
  name: "orderBy"
})
export class OrderByPipe implements PipeTransform {
  transform(array: any, sortBy: string, order?: string): any[] {
    const sortOrder = order ? order : 'asc'; // setting default ascending order

    return orderBy(array, [sortBy], [sortOrder]);
  }
}

@Pipe({
  name: 'dataType',
})
export class DataTypePipe implements PipeTransform {
  constructor(
    private numberFormat: DecimalPipe,
    private dateFormat: dateFormatPipe,
    private dateTimeFormat: dateTimeFormatPipe,
    private timeFormat: DatePipe
  ) {}

  transform(value: any, format: string): any {
    if (format === 'number') return this.numberFormat.transform(value, '1.0-2');
    if (format === 'number2')
      return this.numberFormat.transform(value, '1.2-2');
    if (format === 'number3')
      return this.numberFormat.transform(value, '1.3-3');
    if (format === 'number5')
      return this.numberFormat.transform(value, '1.5-5');
    if (format === 'date') return this.dateFormat.transform(value);
    if (format === 'dateTime') return this.dateTimeFormat.transform(value);
    if (format === 'time')
      return this.dateTimeFormat.transform(value, 'HH:mm:ss');

    return value;
  }
}
