import { Injectable, Inject, Optional } from '@angular/core';
import { Location } from '@angular/common';
import { HttpService } from './http.service';
import { Router, NavigationExtras } from '@angular/router';
import { AuthDecode } from './auth-decode.service';
import { FormRole } from './role-resolver.service';
import { AuthService } from './auth-service.service';
import { Moment } from 'moment';
import * as moment from 'moment';
import { environment } from 'src/environments/environment'
// import { RoleService } from './role.service';

@Injectable()
export class Globals {
  constructor(
    @Optional() private http: HttpService,
    @Optional() private router: Router,
    @Optional() private authDecode: AuthDecode,
    @Optional() private location: Location,
    @Optional() private auth: AuthService
  ) { }

  public DateFormat: string = "dd-MMM-yyyy"
  public DateTimeFormat: string = "dd-MMM-yyyy hh:mm:ss a"
  public DateTimeFormat24H: string = "dd-MMM-yyyy HH:mm:ss"
  private RoleDetail: FormRole;

  findUnique(value, index, self, key) {
    return self.findIndex(x => x[key] === value[key]) === index;
  }

  GoToUrl(url) {
    window.open(url, "_blank");
  }

  public CompareDate(date1: Date, date2: Date): boolean {
    date1 = date1 ? new Date(date1) : null;
    date2 = date2 ? new Date(date2) : null;

    date1 ? date1.setHours(0, 0, 0, 0) : null;
    date2 ? date2.setHours(0, 0, 0, 0) : null;

    return date1 === date2;
  }

  CurrentDate() {

    let d = new Date();
    let Day = d.getDate().toString();
    let Month = d.getMonth() + 1;
    let NewMonth = Month.toString().length < 2 ? '0' + Month.toString() : Month.toString()
    let Year = d.getFullYear().toString();
    let hours = d.getHours();
    let minute = d.getMinutes();
    let seconds = d.getSeconds();

    return Year + NewMonth + Day + '_' + hours + ':' + minute + ':' + seconds;
  }

  GoToReport(reportName: string, parameters: any, newTab: boolean = true, showParam: boolean = false) {
    let navigationExtras: NavigationExtras = {
      queryParams: parameters
    }

    this.http.post("System","ReportingServiceLog",{REPORT_NAME : reportName})
    .subscribe(res =>{})
    // EXCELOPENXML
    // pdf

    if (newTab) {
      let url: string, tree: string;
      if(showParam) {
        tree = this.router.createUrlTree(["report", "p", reportName], navigationExtras).toString();
      } else {
        tree = this.router.createUrlTree(["report", reportName], navigationExtras).toString();
      }

      url = this.location.prepareExternalUrl(tree);

      window.open(url, '_blank');
    } else {
      if(showParam) {
        this.router.navigate(["report","p", reportName], navigationExtras)
      } else {
        this.router.navigate(["report", reportName], navigationExtras)
      }
    }
  }


  ConvertDateToFormatSTD(pDate: Moment) {
    if (pDate && moment(pDate).isValid()) {
      return moment(pDate).format('DD-MMM-YY');
    }
    return null;
  }

  GoToExport(reportName: string, parameters: any, newTab: boolean = true, exportTo: string = '') {
    let navigationExtras: NavigationExtras = {
      queryParams: parameters
    }

    const reportserver: string = environment.report.url;
    const showparameters: string = "false";
    const reportRootPath: string = environment.report.rootPath;
    const fileName = reportName;

    this.http.post("System","ReportingServiceLog",{REPORT_NAME : reportName})
    .subscribe(res =>{})
    const params = this.buildParameterString(parameters);

    const reportUrlStr = reportserver.concat(
      "?",
      "/",
      reportRootPath,
      "/",
      reportName,
      "&rs:Format=" + exportTo,
      "&rs:FileName=" + fileName,
      "&rc:Parameters=",
      showparameters,
      params
    );

    if(newTab) {
      window.open(reportUrlStr, '_blank')
    } else {
      window.open(reportUrlStr, '_self')
    }
  }


  GoToPrint(reportName: string, parameters: any, newTab: boolean = true, exportTo: string = '') {
    let navigationExtras: NavigationExtras = {
      queryParams: parameters
    }

    const reportserver: string = environment.report.url;
    const showparameters: string = "false";
    const reportRootPath: string = environment.report.rootPath;
    const fileName = reportName;
    this.http.post("System","ReportingServiceLog",{REPORT_NAME : reportName})
    .subscribe(res =>{})

    const params = this.buildParameterString(parameters);

    const reportUrlStr = reportserver.concat(
      "?",
      "/",
      reportRootPath,
      "/",
      reportName,
      "&rs:Format=" + exportTo,
      "&rs:FileName=" + fileName,
      "&rc:Parameters=",
      showparameters,
      params
    );

    if (newTab) {
      window.open(reportUrlStr, '_blank')
    } else {
      window.open(reportUrlStr, '_self')
    }
  }

  /**
   * 
   * @param reportName Report name
   * @param parameters Parameters in JSON 
   * @param fileName Filename without extension
   * @param exportTo PDF/pdf or EXCELOPENXML
   */
  GoToExportName(reportName: string, parameters: any, fileName: string = null, exportTo: string = '') {

    const reportserver: string = environment.report.url;
    const showparameters: string = "false";
    const reportRootPath: string = environment.report.rootPath;
    fileName = fileName || reportName;
    this.http.post("System","ReportingServiceLog",{REPORT_NAME : reportName})
    .subscribe(res =>{})

    const params = this.buildParameterString(parameters);

    const reportUrlStr = reportserver.concat(
      "?",
      "/",
      reportRootPath,
      "/",
      reportName,
      "&rs:Format=" + exportTo,
      "&rs:FileName=" + fileName,
      "&rc:Parameters=",
      showparameters,
      params
    );

    window.open(reportUrlStr, '_blank')


  }

  public buildParameterString(parameters): string {
    var parameterString = "";

    for (var param in parameters) {
      if (param) {
        if (parameters[param] instanceof Array === true) {
          for (var arrayParam in parameters[param]) {
            if (arrayParam) {
              parameterString +=
                "&" + param + "=" + parameters[param][arrayParam];
            }
          }
        }
        if (
          parameters[param] != null &&
          parameters[param] instanceof Array === false
        ) {
          parameterString += "&" + param + "=" + parameters[param];
        }
      }
    }
    return parameterString;
  }

  GetTokenDetail(claim: string) {
    let tokenDetail

    tokenDetail = this.authDecode.getDetail(false);

    return tokenDetail[claim];
  }

  SetRoleDetail(role: FormRole) {
    this.RoleDetail = role;
  }

  GetRoleDetail() {
    return this.RoleDetail;
  }


  ClearObject(Object?) {

    for (var key in Object) {
      if (Object.hasOwnProperty(key)) {
        if (Array.isArray(Object[key])) {
          Object[key] = [];
        }
        else if (Number.isInteger(Object[key])) {
          Object[key] = 0;
        }
        else if ((Object[key] instanceof Date)) {
          Object[key] = new Date();
        }
        else {
          Object[key] = null;
        }
      }
    }
    return Object;
  }

  EscapeCharacter(text: string): string {
    let escapeSet = [
      { original: '%', regReplace: /[\%]/g, replaceWith: '[percnt]' },
      { original: '/', regReplace: /[\/]/g, replaceWith: '[slash]' },
      { original: '\\', regReplace: /[\\]/g, replaceWith: '[bslash]' },
      { original: '*', regReplace: /[*]/g, replaceWith: '[star]' }
    ]

    escapeSet.forEach(v => {
      text = text?.toString().replace(v.regReplace, v.replaceWith)
    });

    return text;
  }



  Round(num :number,digits : number){
    return Number(Math.round(parseFloat(num + 'e' + digits)) + 'e-' + digits);
  }

  public CloneDeep(jsonObj: any): any {
    return JSON.parse(JSON.stringify(jsonObj));
  }

}
