import { Component, OnInit, TemplateRef, ContentChildren, ViewChild, ViewContainerRef, ElementRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { ModalService } from '../../../core/modal.service';
import { MatTableDataSource } from '@angular/material/table';
import { MatPaginator } from '@angular/material/paginator';
import { AuthDecode } from 'src/app/core/auth-decode.service';
import { AuthService } from 'src/app/core/auth-service.service';
import * as moment from 'moment';

@Component({
  selector: 'SNEN045',
  templateUrl: './SNEN045.component.html',
  styleUrls: ['./SNEN045.component.scss']
})


export class SNEN045Component implements OnInit  {
  constructor(
    private http: HttpService,
    private router: Router,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private global: Globals,
    private auth: AuthDecode,
    private authService: AuthService,
  ) { }

  public filter: any = {};
  public listCustomer: any[] = [];
  public listOderNo: any[] = [];
  public listCustomer_fLoad: any[] = [];
  public listReason: any[] = [];

  public dataList: any[] = [];
  public dataListSelected: any[] = [];
  public displayedColumns: string[] = [];

  public tableHeader: any[] = [];
  public tableData: any;
  public tableDataSelected: any;
  public params: any = {}

  public formData: any = {};
  public PageCancelOrder: Boolean;
  public Reqcancelorder: Boolean;
  public ssNameTH: string;
  public ssUserName: string;
  public tableCancelOrder: any = [];
  

  @ContentChildren(TemplateRef) templateRef;
  @ViewChild('Paginator1', { static: true }) paginator: MatPaginator;
  @ViewChild('Paginator2', { static: true }) paginator2: MatPaginator;
  @ViewChild('selectList', { static: false }) selectList: ElementRef;

  ngOnInit() {
    this.GetMasterData();
    this.SetTable();
    this.PageCancelOrder = false;
    this.Reqcancelorder = true;
    const show_user: any = [
      this.authService.isAuthenticated() && this.auth.getDetail().ssNameTH,
      this.authService.isAuthenticated() && this.auth.getDetail().ssUserName];
    this.ssNameTH = show_user[0];
    this.ssUserName = show_user[1];

  }

  filterCustomer(event){
    if(!event){
        this.listCustomer = this.listCustomer_fLoad;
    }   
    if (typeof event === 'string') {
        this.listCustomer = this.listCustomer_fLoad.filter(e => e.LOOKUP_CUSTOMER_CODE.toLowerCase().includes(event.toLowerCase())); 
        const customer = this.filter.CUSTOMER_CODE != null? this.filter.CUSTOMER_CODE : '';
        this.filter.IS_MATCH_CUSTOMER_FILTER = this.listCustomer.some(e => e.LOOKUP_CUSTOMER_CODE.toLowerCase().includes(customer.toLowerCase()));
    } 
  }      

  onChangeofCustomer(event){
    this.filter.IS_MATCH_CUSTOMER_FILTER = true;
  }

  private SortData(){
    this.dataList.sort((a, b) => (a.ORDER_DATE > b.ORDER_DATE) ? -1 : 1);
    this.dataListSelected.sort((a, b) => (a.ORDER_DATE > b.ORDER_DATE) ? -1 : 1);
  }
  
  private SetTable(){
    this.tableData = null;
    this.tableDataSelected = null;

    this.tableData = new MatTableDataSource<any>(this.dataList || []);
    this.tableData.paginator = this.paginator;
    this.tableDataSelected = new MatTableDataSource<any>(this.dataListSelected || []);
    this.tableDataSelected.paginator = this.paginator2;
  }

  //Cancel Order Main 
  public GetOrderNo() {
    this.listOderNo = [];
    //const param = {
    //  CUSTOMER_CODE: this.params.Customer_Code = "008114A",
    //};
    this.http.post("PC", "SNEN045_GetOrderno", null)
      .subscribe(res => {
        this.listOderNo = res.Table;
        console.log(res.Table)
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });


  }

  private GetMasterData() {

    this.http.get('PC', 'SNEN045_GetCustomer')
      .subscribe(res => {
        if(res.result == "Done"){
          this.listCustomer = res.set.Table;
          this.listCustomer_fLoad = res.set.Table;
        }
        else{
          this.modal.error(res.message, this.viewref)
        }
      },(err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
    });

    this.http.get('PC', 'SNEN045_GetReason')
      .subscribe(res => {
        if(res.result == "Done"){
          this.listReason = res.set.Table;
        }
        else{
          this.modal.error(res.message, this.viewref)
        }
      },(err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
    });


    this.http.get('System', 'GetHeaderGridView', 'SNEN045')
      .subscribe((result) => {
        this.tableHeader = result.filter((v) => v.VISIBILITY === true);
        this.displayedColumns = this.tableHeader.map(v => v.FIELD_NAME);
    });

  }
  
  private GetPartData(){
    const params = {
      CUSTOMER_CODE : this.filter.CUSTOMER_CODE,
      ORDER_NO: this.filter.ORDER_NO,
      ORDER_PART_NO: this.filter.ORDER_PART_NO,
    }
    this.http.post('PC', 'SNEN045_GetPartData', params)
      .subscribe(res => {
        console.log(res);
        
        if(res.result == "Done"){
          if(this.formData.CUSTOMER_CODE == this.filter.CUSTOMER_CODE){
            const list = res.set.Table.filter((el) => !this.dataListSelected.find(item => (item.ORDER_NO + item.ORDER_PART_NO) === (el.ORDER_NO + el.ORDER_PART_NO) ));

            this.dataList = [];
            this.dataList = list;
            this.dataList.sort((a, b) => (a.ORDER_DATE > b.ORDER_DATE) ? -1 : 1);
            this.tableData = new MatTableDataSource<any>(this.dataList || []);
            this.tableData.paginator = this.paginator;
          }
          else{
            this.formData.CUSTOMER_CODE = this.filter.CUSTOMER_CODE;
            const list = res.set.Table;

            this.dataList = [];
            this.dataListSelected = [];
            this.dataList = list;
            this.dataList.sort((a, b) => (a.ORDER_DATE > b.ORDER_DATE) ? -1 : 1);
            this.SetTable();
          }
        }
        else{
          this.modal.error(res.message, this.viewref)
        }
      },(err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public Search() {
    if(!this.filter.CUSTOMER_CODE || !this.filter.IS_MATCH_CUSTOMER_FILTER){
      this.modal.warning("กรุณาเลือก Customer", this.viewref);
      return;
    }
    if(this.dataListSelected.length > 0){
      if(this.formData.CUSTOMER_CODE == this.filter.CUSTOMER_CODE){
        this.GetPartData();
      }
      else{
        this.modal.confirm(
          "คุณมีรายการที่ทำค้างไว้อยู่ หากดำเนินการค้นหาใหม่ข้อมูลจะไม่ได้ถูกบันทึกไว้ \n คุณแน่ใจที่จะต้องการค้นหาข้อมูลใหม่ใช่ไหม?",
          this.viewref,
          () => {
            this.GetPartData();
          },
          () => {}
        );
      }
    }
    else{
      this.GetPartData();
    }
  }

  public ToggleToSelect(data) {
    const orderno = (data.ORDER_NO != null ? data.ORDER_NO : "");
    const partno = (data.ORDER_PART_NO != null ? data.ORDER_PART_NO : "");
    const rowId =  orderno + "_" + partno;
    const checkboxId = "chk_" + orderno + "_" + partno;
    let row = document.getElementById(rowId);
    let checkbox  = document.getElementById(checkboxId) as HTMLInputElement | null;
    if (!row.classList.contains('selectdata')) {
      row.classList.add('selectdata');
      data.CHECKBOX = true;
      checkbox.checked = true;
    } 
    else {
      row.classList.remove('selectdata');
      data.CHECKBOX = null;
      checkbox.checked = false;
    }
  }

  public AddSelectedPart(){
    if(this.dataList.some(e => e.CHECKBOX == true)){
      const remain = this.dataList.filter(e => e.CHECKBOX != true);
      const selected = this.dataList.filter(e => e.CHECKBOX == true);

      this.dataList = remain;
      this.dataListSelected = [ ...this.dataListSelected, ...selected ];
      this.SortData();
      this.SetTable();
    }
  }
  
  public RemoveSelectedPart(row){
    const thisItem = this.dataListSelected.find(data => (data.ORDER_NO + "_" + data.ORDER_PART_NO) === (row.ORDER_NO + "_" + row.ORDER_PART_NO));
    const thisIndex = this.dataListSelected.indexOf(thisItem);

    this.dataList.push(thisItem);
    this.dataListSelected.splice(thisIndex, 1);
    this.SortData();
    this.SetTable();
  }

  public ConfirmSelectedPart(){
    this.modal.confirm(
      "คุณต้องการบันทึกการทำงานใช่หรือไม่",
      this.viewref,
      () => {
        this.formData.REASON_TEXT = this.listReason.find((e)=> e.REASON_CODE.toString() === this.formData.REASON_CODE).REASON_TEXT;
        this.formData.DETAIL = this.dataListSelected;

        this.http.post("PC", "SNEN045_SaveCancelPart", this.formData).subscribe(
          (res) => {
            if (res.result === "DONE") {
              this.modal.info(`บันทึกเสร็จสิ้น`, this.viewref, () => {});
              this.dataListSelected = [];
              this.tableDataSelected = new MatTableDataSource<any>(this.dataListSelected || []);
              this.tableDataSelected.paginator = this.paginator2;
              this.GetPartData();
            } 
            else if (res.result === "FAIL") {
              this.modal.error(res.message, this.viewref);
            } 
            else {
              this.modal.error("พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",this.viewref);
            }
          },
          (err) => {
            this.modal.error("พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",this.viewref);
          }
        );
      },
      () => {}
    );

  }

  public Back() {
    this.PageCancelOrder = true;
    this.Reqcancelorder = false;
  }

  public GotoRequestCancel() {
    this.PageCancelOrder = false;
    this.Reqcancelorder = true;
    this.formData = {};
  }

  public Report(){
    const params = {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_DATA_MONTH: moment().format("YYYYMM"),
    }

    this.global.GoToReport(
      "PMS_RPT_EN_SNEN045_CANCEL_ORDER" 
      ,params
      ,true
      ,true
    );
  }

}
