import { Component, OnInit, ViewContainerRef } from '@angular/core'
import { HttpService } from 'src/app/core/http.service';
import { ModalService } from 'src/app/core/modal.service';
import * as moment from 'moment';
import { Globals } from 'src/app/core/global';
import { SelectionModel } from '@angular/cdk/collections';

@Component({
  selector: 'snse033_w',
  templateUrl: './SNSE033_W.component.html',
  styleUrls: ['./SNSE033_W.component.css']
})

export class SNSE033_WComponent implements OnInit {

  constructor(
    private http: HttpService,
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private global: Globals
  ) { }


  public tableDataAir: any = [];
  public tableDataSea: any = [];

  public paramAir: any = {};
  public paramSea: any = {};

  public dataAir: any = {};
  public dataSea: any = {};
  public dataSet: any = {};

  public currentDate: moment.Moment = moment();

  public selection = new SelectionModel<any>(true, []);

  public selection2 = new SelectionModel<any>(true, []);

  public DataIsSelectedAir: any = [];

  public DataIsSelectedSea: any = [];

  public tblFooter: any = [
    {
      fieldName: "PACK_BY",
      value: "Total",
      show: true
    },
    {
      fieldName: "GROSS_WEIGHT",

      show: true
    },
    {
      fieldName: "GROSS_MM",

      show: true
    },
    {
      fieldName: "PACK_QTY",

      show: true
    },
    {
      fieldName: "AMOUNT",

      show: true
    }

  ]

  public lookupCustomerCodeAir: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE033_W_GetCustomerCodeAir",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupPreInvoiceAir: any = {
    ColName: "PINVOICE_TEXT",
    PlaceholderText: "Select Pre-invoice no.",
    SourceController: "SL",
    SourceMethodName: "SNSE033_W_GetPreInvoiceAir",
    NoOfShowRecords: 20,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      CUSTOMER_CODE: "",
      BRANCH_CODE: "01"
    }
  }

  public lookupCustomerCodeSea: any = {
    ColName: "LOOKUP_CUSTOMER_CODE",
    PlaceholderText: "Select Customer Code",
    SourceController: "SL",
    SourceMethodName: "SNSE033_W_GetCustomerCodeSea",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public lookupPreInvoiceSea: any = {
    ColName: "PINVOICE_TEXT",
    PlaceholderText: "Select Pre-invoice no.",
    SourceController: "SL",
    SourceMethodName: "SNSE033_W_GetPreInvoiceSea",
    NoOfShowRecords: 20,
    MinLengthSearch: 0,
    UpperCase: true,
    SourceParams: {
      CUSTOMER_CODE: "",
      BRANCH_CODE: ""
    }
  }

  ngOnInit() {
    this.paramAir.ATTACH_SHEET = 'Y';
    this.paramSea.ATTACH_SHEET = 'Y';
  }

  public SetSelectedCustomerCode(values: any, pSendBy: any) {
    if (pSendBy == 'A') {
      this.paramAir.CUSTOMER_CODE = values.CUSTOMER_CODE;
      this.paramAir.CUSTOMER_NAME = values.CUSTOMER_NAME;
      this.paramAir.BRANCH_CODE = values.BRANCH_CODE;
      this.paramAir.BRANCH_NAME = values.BRANCH_NAME;

      this.dataAir.INVOICE_NO = '';

      this.lookupPreInvoiceAir.SourceParams.CUSTOMER_CODE = values.CUSTOMER_CODE;
      this.lookupPreInvoiceAir.SourceParams.BRANCH_CODE = values.BRANCH_CODE;


      this.paramAir.PINVOICE_NO = '';
      this.paramAir.PINVOICE_DATE = '';
      this.tableDataAir = [];
    } else {

      this.paramSea.CUSTOMER_CODE = values.CUSTOMER_CODE;
      this.paramSea.CUSTOMER_NAME = values.CUSTOMER_NAME;
      this.paramSea.BRANCH_CODE = values.BRANCH_CODE;
      this.paramSea.BRANCH_NAME = values.BRANCH_NAME;

      this.dataSea.INVOICE_NO = '';

      this.lookupPreInvoiceSea.SourceParams.CUSTOMER_CODE = values.CUSTOMER_CODE;
      this.lookupPreInvoiceSea.SourceParams.BRANCH_CODE = values.BRANCH_CODE;


      this.paramSea.PINVOICE_NO = '';
      this.paramSea.PINVOICE_DATE = '';
      this.tableDataSea = [];
    }

  }

  public SelectPreInvoice(values: any, pSendBy: any) {

    if (pSendBy == 'A') {
      this.paramAir.PINVOICE_NO = values.PINVOICE_NO;
      this.paramAir.PINVOICE_DATE = values.PINVOICE_DATE;
      this.paramAir.DOC_TYPE = values.DOC_TYPE;
    } else {
      this.paramSea.PINVOICE_NO = values.PINVOICE_NO;
      this.paramSea.PINVOICE_DATE = values.PINVOICE_DATE;
      this.paramSea.DOC_TYPE = values.DOC_TYPE;
    }

    this.GetData(pSendBy);
  }

  private GetData(pSendBy: any) {

    if (pSendBy == 'A') {
      this.tableDataAir = [];
      this.selection.clear();
      this.DataIsSelectedAir = [];
      this.http.post('SL', 'SNSE033_W_GetDataAir', this.paramAir)
        .subscribe(res => {
          this.tableDataAir = res.Table1;
        })
    } else {
      this.tableDataSea = [];
      this.selection2.clear();
      this.DataIsSelectedSea = [];
      this.http.post('SL', 'SNSE033_W_GetDataSea', this.paramSea)
        .subscribe(res => {
          this.tableDataSea = res.Table1;
        })
    }
  }

  public Confirm_Click(pSendBy: any) {

    var msg: any = '';

    if (pSendBy == 'A') {


      if (this.paramAir.EXCHANGE_RATE == '') {
        msg = "Exchange Rate Invalid." + '<br/>';
      }


      if (this.dataAir.FLIGHT_NO == '' || this.dataAir.FLIGHT_NO == undefined) {
        msg = msg + "Flight No. is not input.." + "<br/>";
      }
      if (this.dataAir.AWB_NO == '' || this.dataAir.AWB_NO == undefined) {
        msg = msg + "AWB No. is not input.." + "<br/>";
      }

      var dToDate = moment().locale("en").startOf('day');
      if (this.dataAir.AWB_DATE == '' || this.dataAir.AWB_DATE == undefined) {
        msg = msg + "AWB Date. is not input.." + "<br/>";
      } else if (this.dataAir.AWB_DATE <= dToDate) {
        msg = msg + "AWB Date. is lower than current date" + "<br/>";
      }

      var thai = /[\u0E01-\u0E5B]/;
      if (thai.test(this.dataAir.FLIGHT_NO)) {
        msg = msg + "Flight No. is cannot input thai language " + "<br/>";
	this.dataAir.FLIGHT_NO = '' ;
      }
      if (thai.test(this.dataAir.AWB_NO)) {
        msg = msg + "AWB No. is cannot input thai language " + "<br/>";
	this.dataAir.AWB_NO = '' ;
      }
      if (msg != '') {
        this.modal.warning(msg, this.viewRef);
        return;
      }

      var nCT = 0;
      var QtyEmpty = 0;


      if (this.DataIsSelectedAir.length == 0) {
        this.modal.warning('No "Case No." to be selected. Please check', this.viewRef);
        return;
      }

      this.DataIsSelectedAir.forEach((value) => {
        if (value.CONTAINER_NO == '') {
          nCT = nCT + 1;
        }
        if (value.PACK_QTY <= 0) {
          QtyEmpty = QtyEmpty + 1;
        }
      });

      if (nCT != 0) {
        this.modal.warning('Container No. Invalid.', this.viewRef);
        return;
      }
      if (QtyEmpty != 0) {
        this.modal.warning('Pack qty cannot be 0. Please check', this.viewRef);
        return;
      }
      this.DataIsSelectedAir[0].AWB_NO = this.dataAir.AWB_NO;
      this.DataIsSelectedAir[0].AWB_DATE = this.dataAir.AWB_DATE;
      this.DataIsSelectedAir[0].FRC = this.dataAir.FRC;
      this.DataIsSelectedAir[0].FWC = this.dataAir.FWC;
      this.DataIsSelectedAir[0].INS = this.dataAir.INS;
      this.DataIsSelectedAir[0].FLIGHT_NO = this.dataAir.FLIGHT_NO;
      this.DataIsSelectedAir[0].TOTAL_CASE = this.DataIsSelectedAir.length;
      //console.log('dataSelected[0] : ', dataSelected[0]);
      this.http.post('SL', 'CheckIssueInvoiceRunning', this.DataIsSelectedAir[0])
        .subscribe(res => {
          if (res.status == 'fail') {
            this.modal.warning(res.massage, this.viewRef);
          }
          else {
      this.modal.confirm("Are you sure to confirm this invoice?", this.viewRef, () => {
      this.http.post('SL', 'SNSE033_W_ConfirmAir', this.DataIsSelectedAir)
        .subscribe(result => {
          if (result.substring(0, 4) == "DONE") {
            this.GetData(pSendBy);
            this.dataAir.INVOICE_NO = result.substring(5, 20);
            this.modal.info("Confirm successfully [Invoice no.] : " + result.substring(5, 20), this.viewRef);

            this.dataAir.AWB_NO = '';
            this.dataAir.AWB_DATE = '';
            this.dataAir.FRC = '';
            this.dataAir.FWC = '';
            this.dataAir.INS = '';
            this.dataAir.FLIGHT_NO = '';

            if (this.paramAir.ATTACH_SHEET == 'Y') {
              this.OpenReport('FA', result.substring(5, 20))
            }
          } else {
            this.GetData(pSendBy);
            this.modal.warning("Found an error, please contract support team", this.viewRef);
          }
        }, (err) => {
          this.GetData(pSendBy);
          console.log(err, 'err')
          this.modal.warning("Found an error, please contract support team", this.viewRef);
        })
      });
          }
        }, (err) => {
          console.log(err, 'err')
          this.modal.warning("Found an error, please contract support team", this.viewRef);
        })
    }


    else { //------------- Sea
      msg = '';
      if (!this.dataSea.VESSEL) {
        msg = msg + "Vessel is not input.." + "<br/>";
      }
      if (!this.dataSea.BL_NO) {
        msg = msg + "B/L No. is not input.." + "<br/>";
      }
      if (!this.dataSea.BL_DATE) {
        msg = msg + "B/L Date is not input.." + "<br/>";
      }
      //---- Compare date
      var dToDate = moment().locale("en").startOf('day');
      if (this.dataSea.BL_DATE <= dToDate) {
        msg = msg + "B/L Date is lower than current date" + "<br/>";
      }

      var thai = /[\u0E01-\u0E5B]/;
      if (thai.test(this.dataSea.VESSEL)) {
        msg = msg + "Vessel is cannot input thai language " + "<br/>";
	this.dataSea.VESSEL = '' ;
      }
      if (thai.test(this.dataSea.BL_NO)) {
        msg = msg + "B/L No. is cannot input thai language " + "<br/>";
	this.dataSea.BL_NO = '' ;
      }
      if (msg != '') {
        this.modal.warning(msg, this.viewRef);
        return;
      }

      //var dataSelected = this.tableDataSea.filter(v => v.SELECT_CONFIRM == true);
      var nCT = 0;
      var QtyEmpty = 0;
      if (this.DataIsSelectedSea.length == 0) {
        this.modal.warning('No "Case No." to be selected. Please check', this.viewRef);
        return;
      }

      this.DataIsSelectedSea.forEach((value) => {
        if (value.CONTAINER_NO == '') {
          nCT = nCT + 1;
        }
        if (value.PACK_QTY <= 0) {
          QtyEmpty = QtyEmpty + 1;
        }
      });

      if (nCT != 0) {
        this.modal.warning('Container No. Invalid.', this.viewRef);
        return;
      }
      if (QtyEmpty != 0) {
        this.modal.warning('Pack qty cannot be 0. Please check', this.viewRef);
        return;
      }
      this.DataIsSelectedSea[0].BL_NO = this.dataSea.BL_NO;
      this.DataIsSelectedSea[0].BL_DATE = this.dataSea.BL_DATE;

      this.DataIsSelectedSea[0].FRC = this.dataSea.FRC;
      this.DataIsSelectedSea[0].FWC = this.dataSea.FWC;
      this.DataIsSelectedSea[0].INS = this.dataSea.INS;
      this.DataIsSelectedSea[0].VESSEL = this.dataSea.VESSEL;
      this.DataIsSelectedSea[0].TOTAL_CASE = this.DataIsSelectedSea.length;

      this.http.post('SL', 'CheckIssueInvoiceRunning', this.DataIsSelectedSea[0])
        .subscribe(res => {
          if (res.status == 'fail') {
            this.modal.warning(res.massage, this.viewRef);
          }
          else {
      this.modal.confirm("Are you sure to confirm this invoice?", this.viewRef, () => {
      this.http.post('SL', 'SNSE033_W_ConfirmSea', this.DataIsSelectedSea)
        .subscribe(result => {
          if (result.substring(0, 4) == "DONE") {
            this.GetData(pSendBy);
            this.dataSea.INVOICE_NO = result.substring(5, 20);
            this.modal.info("Confirm successfully [Invoice no.] : " + result.substring(5, 20), this.viewRef);
            this.dataSea.BL_NO = '';
            this.dataSea.BL_DATE = '';
            this.dataSea.FRC = '';
            this.dataSea.FWC = '';
            this.dataSea.INS = '';
            this.dataSea.VESSEL = '';
            if (this.paramSea.ATTACH_SHEET == 'Y') {
              this.OpenReport('FS', result.substring(5, 20))
            }
          } else {
            this.GetData(pSendBy);
            this.modal.warning("Found an error, please contract support team", this.viewRef);
          }
        }, (err) => {
          this.GetData(pSendBy);
          this.modal.warning("Found an error, please contract support team", this.viewRef);
        })
      });
    }
        }, (err) => {
          this.modal.warning("Found an error, please contract support team", this.viewRef);
        })
    }
  }

  public CheckPrintChangeAir(value) {
    if (value.checked == true) {
      this.paramAir.ATTACH_SHEET = 'Y'
    }
    else {
      this.paramAir.ATTACH_SHEET = 'N'
    }
  }

  public CheckPrintChangeSea(value) {
    if (value.checked == true) {
      this.paramSea.ATTACH_SHEET = 'Y'
    }
    else {
      this.paramSea.ATTACH_SHEET = 'N'
    }
  }

  private OpenReport(docType, docNo) {
    this.global.GoToReport("PMS_RPT_SALES_SNSE044R_N", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_DOC_TYPE: docType,
      P_DOC_NO_S: docNo,
      P_TYPE: "ISSUE"
    })
  }

  isAllSelected() {
    const numSelected = this.selection.selected.length;
    const numRows = this.tableDataAir.length;
    return numSelected === numRows;
  }

  Toggle() {
    if (this.isAllSelected()) {
      this.selection.clear()
      this.tableDataAir.map(x => {
        this.DataIsSelectedAir = [];
      })
    } else {
      this.tableDataAir.forEach(row => this.selection.select(row));
      this.DataIsSelectedAir = [];
      this.tableDataAir.map(x => {
        this.DataIsSelectedAir.push(x);
      })
    }
  }

  CheckboxChange(event, data) {
    if (event.checked) {
      this.DataIsSelectedAir.push(data);
    }
    else {
      this.DataIsSelectedAir.splice(this.DataIsSelectedAir.indexOf(data), 1);

    }
  }





  isAllSelected2() {
    const numSelected = this.selection2.selected.length;
    const numRows = this.tableDataSea.length;
    return numSelected === numRows;
  }

  Toggle2() {
    if (this.isAllSelected2()) {
      this.selection2.clear()
      this.tableDataSea.map(x => {
        this.DataIsSelectedSea = [];
      })
    } else {
      this.tableDataSea.forEach(row => this.selection2.select(row));
      this.DataIsSelectedSea = [];
      this.tableDataSea.map(x => {
        this.DataIsSelectedSea.push(x);
      })
    }
  }

  CheckboxChange2(event, data) {
    if (event.checked) {
      this.DataIsSelectedSea.push(data);
    }
    else {
      this.DataIsSelectedSea.splice(this.DataIsSelectedSea.indexOf(data), 1);
    }
  }
}
