import { SelectionModel } from "@angular/cdk/collections";
import {
  Component,
  ElementRef,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { FormControl, Validators } from "@angular/forms";
import { Globals } from "src/app/core/global";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";

@Component({
  selector: "SNWR019",
  templateUrl: "SNWR019.component.html",
})
export class SNWR019Component implements OnInit {
  constructor(
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private http: HttpService,
    private el: ElementRef,
    private global: Globals
  ) { }

  @ViewChild("dialogAdd", { static: false }) dialogAdd: any;
  @ViewChild("dialog", { static: false }) dialog: any;
  @ViewChild("file") file: any;

  public paramSearch: any = {};
  public paramAdd: any = {};
  public dataCheckStock: any = {};
  public tableHeader: any = [];
  public isSavePage: boolean = false;
  public checkType: any = [
    { ID: 1, TYPE: "CHECK_STOCK", DESC: "Check stock" },
    { ID: 2, TYPE: "CYCLE_COUNT", DESC: "Cycle count" },
  ];

  public dataStock: any = [];
  public dataPreview: any = [];
  public error_list: any = [];
  public files: any = [];
  public duplicateParts: any;
  public total_error: number;
  public FileName: string = "";
  public selection = new SelectionModel<any>(true, []);
  public hideColumns: string[];

  public lookupConfig: any = {
    ColName: "LOOKUP_PART_NAME_TH",
    PlaceholderText: "Select Part No.",
    SourceController: "wPMS",
    SourceMethodName: "GetPartNo",
    NoOfShowRecords: 30,
    MinLengthSearch: 3,
    UpperCase: true,
  };

  //Validate
  SetCheckStockDateControl = new FormControl("", [Validators.required]);
  SetCheckStockTypeControl = new FormControl("", [Validators.required]);

  public ngOnInit() {
    this.Search();
  }

  public Search() {
    this.http
      .post("WH", "SNWR019_GetData", this.paramSearch)
      .subscribe((res) => {
        this.tableHeader = res.Table;
      });
  }

  CheckInit() {
    this.dataStock.push({
      PART_NO: null,
      PART_NAME_EN: null,
      PART_NAME_TH: null,
      COUNT_QTY: null,
    });
  }

  public DownLoadTemplate() {
    let link = "assets/templates/SNWR019_Upload_Template.xlsx";
    window.open(link, "_blank");
  }

  public fileChange(fileInput) {
    this.selection.clear();
    this.files = fileInput.target.files;
    this.FileName = this.files[0].name;

    this.UpLoad();
  }

  private UpLoad() {
    //this.dataStock = [];
    let formData = new FormData();
    formData.append("file", this.files[0]);

    this.http
      .postFile(
        "WH",
        `SNWR019P_UploadExcel/${this.dataCheckStock.HEADER_ID.toString()}`,
        formData
      )
      .subscribe(
        (res) => {
          this.total_error = 0;
          this.dataStock = res.filter((x) => x.PART_NO);
          let countrow = 0;
          this.total_error = this.dataStock.filter((x, i) => {
            if (x.ERR_FLAG == "Y") {
              countrow += 1;
              this.error_list.push({
                Records: i + 1,
              });
            }
          });
          this.total_error = countrow;
        },
        (err) => {
          this.modal.error("Upload error", this.viewRef);
        }
      );
  }

  public IsCheckPage(data: any) {
    this.dataStock = [];
    this.dataPreview = [];
    this.error_list = [];
    this.files = [];
    this.total_error = 0;
    this.FileName = "";
    this.CheckInit();
    this.GetDataByID(data);
    this.dataCheckStock = data;
    this.isSavePage = !this.isSavePage;
  }

  public GetDataByID(data) {
    console.log("data", data);

    this.http
      .get("WH", "SNWR019_GetData_ByID", data.HEADER_ID)
      .subscribe((res) => {
        this.dataPreview = res.Table;
        if (data.CHECK_TYPE === "CYCLE_COUNT")
          this.hideColumns = ["DELETE", "MESSAGE"];
        if (data.CHECK_TYPE === "CHECK_STOCK")
          this.hideColumns = [
            "DELETE",
            "MESSAGE",
            "UNPACK_QTY",
            "SHELVE_QTY",
            "PL_QTY",
            "N3_QTY",
            "N2_QTY",
            "N1_QTY",
            "N_QTY",
          ];
      });
  }

  public OpenFileBrowser() {
    this.file.nativeElement.value = "";
    this.files = [];
    this.FileName = "";
    this.file.nativeElement.click();
  }

  public Back() {
    this.isSavePage = !this.isSavePage;
  }

  public CreateCheckStock() {
    this.paramAdd = {};
    this.dialogAdd.OpenDialog();
  }

  public SaveAdd() {
    if (this.paramAdd.CHECK_DATE && this.paramAdd.CHECK_TYPE) {
      this.modal.confirm(
        "Do you want to save change you have made?",
        this.viewRef,
        this.ConfirmSave
      );
    } else {
      this.modal.warning("Please Input fill", this.viewRef);
    }
  }

  public ConfirmSave = () => {
    this.http.post("WH", "SNWR019_INIT", this.paramAdd).subscribe(
      (res) => {
        if (res.Table[0].STATUS == "Done") {
          this.modal.info(
            "Transaction complete: applied and saved.",
            this.viewRef
          );
          this.Search();
          this.paramAdd = this.global.ClearObject(this.paramAdd);
          this.dialogAdd.Close();
          this.SetCheckStockDateControl.reset();
          this.SetCheckStockTypeControl.reset();
        } else if (res.Table[0].STATUS == "Duplicate") {
          this.modal.warning(
            "No change to save. Please submit before create new round.",
            this.viewRef
          );
        } else {
          this.modal.error(
            "No change to save. Something went wrong.",
            this.viewRef
          );
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team",
          this.viewRef
        );
      }
    );
  };

  public CancelAdd() {
    this.dialogAdd.Close();
  }

  //Check Part QTY
  public SetPartNo(data, event) {
    if (event) {
      this.CheckDuplicatePartNo(event.PART_NO);
      if (this.duplicateParts.length > 0) {
        this.modal.warning(
          `Cannot use this part, Part No ${event.PART_NO} is duplicate`,
          this.viewRef
        );
        data.PART_NO = null;
        data.PART_NAME_EN = null;
        data.PART_NAME_TH = null;
      } else {
        data.PART_NO = event.PART_NO;
        data.PART_NAME_EN = event.PART_NAME_EN;
        data.PART_NAME_TH = event.PART_NAME_TH;
        data.ERR_FLAG = "N";
        data.ERR_FLAG_PART = "N";
        data.MSG = null;

        this.error_list = [];
        let countrow = 0;
        this.total_error = this.dataStock.filter((x, i) => {
          if (x.ERR_FLAG == "Y") {
            countrow += 1;
            this.error_list.push({
              Records: i + 1,
            });
          }
        });
        this.total_error = countrow;
      }
    }
  }

  public onInputCountQty(data) {
    if (
      data.COUNT_QTY &&
      data.ERR_FLAG_CQ == "Y" &&
      data.ERR_FLAG_PART == "N"
    ) {
      data.ERR_FLAG = "N";
      data.ERR_FLAG_CQ = "N";
      data.MSG = null;

      this.error_list = [];
      let countrow = 0;
      this.total_error = this.dataStock.filter((x, i) => {
        if (x.ERR_FLAG == "Y") {
          countrow += 1;
          this.error_list.push({
            Records: i + 1,
          });
        }
      });
      this.total_error = countrow;
    }
  }

  public AddRecord() {
    this.dataStock.push({
      PART_NO: null,
      PART_NAME_EN: null,
      PART_NAME_TH: null,
      COUNT_QTY: null,
    });
    this.dataStock = this.dataStock.slice();
  }

  public RemoveRecord(index) {
    this.dataStock.splice(index, 1);
    this.dataStock = this.dataStock.slice();

    this.error_list = [];
    let countrow = 0;
    this.total_error = this.dataStock.filter((x, i) => {
      if (x.ERR_FLAG == "Y") {
        countrow += 1;
        this.error_list.push({
          Records: i + 1,
        });
      }
    });
    this.total_error = countrow;
  }

  public CheckDuplicatePartNo(partNo?: any) {
    this.duplicateParts = [];

    if (partNo) {
      const foundDuplicate =
        this.dataStock.filter((x) => x.PART_NO && x.PART_NO === partNo).length >
        1;
      if (foundDuplicate) {
        this.duplicateParts.push(partNo);
      }
    } else {
      this.duplicateParts = this.dataStock
        .filter(
          (item) =>
            this.dataStock.filter(
              (x) => x.PART_NO && x.PART_NO === item.PART_NO
            ).length > 1
        )
        .map((item) => item.PART_NO);
      this.duplicateParts = [...new Set(this.duplicateParts)];
    }
  }

  public SaveCheckStockAdd() {
    if (this.dataStock.length > 0 && this.dataCheckStock.HEADER_ID) {
      this.modal.confirm(
        "Do you want to save change you have made?",
        this.viewRef,
        this.SaveCheckStock
      );
    } else {
      this.modal.warning(
        "Don't have data for save, please try again.",
        this.viewRef
      );
    }
  }

  public SaveCheckStock = () => {
    this.CheckDuplicatePartNo();
    if (this.duplicateParts.length > 0) {
      var partWarning = this.duplicateParts.concat().join(", ");
      this.modal.warning(
        `No change to save, Part No ${partWarning} is duplicate`,
        this.viewRef
      );
    } else {
      const saveData = this.dataStock.filter((part) => part.PART_NO);
      if (saveData.length > 0) {
        this.http
          .post("WH", ["SNWR019_Save", this.dataCheckStock.HEADER_ID], saveData)
          .subscribe(
            (res) => {
              if (res === "Done") {
                this.modal.info(
                  "Transaction complete: applied and saved.",
                  this.viewRef
                );
                this.Search();
                this.GetDataByID(this.dataCheckStock);
              }
            },
            (err) => {
              this.modal.warning(
                "Found an error, please contract support team",
                this.viewRef
              );
            }
          );
      } else {
        this.modal.warning(`No change to save.`, this.viewRef);
      }
    }
  };

  public SubmitAdd() {
    if (this.dataPreview.length > 0 && this.dataCheckStock.HEADER_ID) {
      this.modal.confirm(
        "Do you want to submit check round?",
        this.viewRef,
        this.ConfirmSubmit
      );
    } else {
      this.modal.warning(
        "Don't have data for submit, please try again.",
        this.viewRef
      );
    }
  }

  public ConfirmSubmit = () => {
    this.http.post("WH", "SNWR019_SUBMIT", this.dataCheckStock).subscribe(
      (res) => {
        if (res == "Done") {
          this.modal.info(
            "Transaction complete: applied and saved.",
            this.viewRef
          );
          this.Search();
          this.Back();
        } else {
          this.modal.error(
            "No change to save. Something went wrong.",
            this.viewRef
          );
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team",
          this.viewRef
        );
      }
    );
  };

  public PrintCountReport() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR019_COUNT", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_ID: this.dataCheckStock.HEADER_ID,
      P_SHOW_ALL: "Y",
    });
  }
}
