import { Component, OnInit, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { Globals } from "src/app/core/global";

@Component({
  selector: "SNWR028",
  templateUrl: "SNWR028.component.html",
})
export class SNWR028Component implements OnInit {
  constructor(
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private http: HttpService,
    private global: Globals
  ) {}

  public params: any = {};
  public tableHeader: any = [];
  public caseNoFrom: any;
  public caseNoTo: any;

  public ngOnInit() {
    this.params.PRINT_NUMBER = 0;
  }

  public PrintCaseNumber() {
    if (this.params.PRINT_NUMBER < 1) {
      this.modal.warning("Package QTY must be more than 1.", this.viewRef);
    } else {
      this.http
        .post("WH", "SNWR028_Print_Case_Number", this.params)
        .subscribe((res) => {
          if (res.Result == "Done") {
            this.caseNoFrom = res.CaseNoFrom;
            this.caseNoTo = res.CaseNoTo;
            this.modal.info(
              `Print Case Number ${res.CaseNoFrom} - ${res.CaseNoTo}, Successfully.`,
              this.viewRef,
              () => {
                this.Print();
                this.params.PRINT_NUMBER = 0;
              }
            );
          }
        });
    }
  }

  public Print() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR028_CASE", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_CASE_NO_FROM: this.caseNoFrom,
      P_CASE_NO_TO: this.caseNoTo,
    });
  }

  public FindCaseNo() {
    this.http.post("WH", "SNWR028_Get_Case_Number", this.params).subscribe(
      (res) => {
        if (res.Table[0]) {
          const case_no = res.Table[0].CASE_NO;
          this.global.GoToReport("PMS_RPT_WMS_SNWR028_CASE", {
            P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
            P_CASE_NO_FROM: case_no,
            P_CASE_NO_TO: case_no,
          });
        } else {
          this.modal.error("Unavailable Case Number.", this.viewRef);
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team",
          this.viewRef
        );
      }
    );
  }
}
