import { Component, OnInit, ViewContainerRef } from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { Globals } from "src/app/core/global";

@Component({
  selector: "SNWR030",
  templateUrl: "SNWR030.component.html",
})
export class SNWR030Component implements OnInit {
  constructor(
    private modal: ModalService,
    private viewRef: ViewContainerRef,
    private http: HttpService,
    private global: Globals
  ) {}

  public params: any = {};
  public vanNoFrom: any;
  public vanNoTo: any;

  public ngOnInit() {
    this.params.PRINT_NUMBER = 0;
  }

  public PrintVanNumber() {
    if (this.params.PRINT_NUMBER < 1) {
      this.modal.warning("Van QTY must be more than 0.", this.viewRef);
    } else {
      this.http
        .post("WH", "SNWR030_Print_Van_Number", this.params)
        .subscribe((res) => {
          if (res.Result == "Done") {
            this.vanNoFrom = res.VanNoFrom;
            this.vanNoTo = res.VanNoTo;
            this.modal.info(`Print Van Number ${res.VanNoFrom} - ${res.VanNoTo}, Successfully.`,this.viewRef,
              () => {
                this.Print();
                this.params.PRINT_NUMBER = 0;
              }
            );
          }
        });
    }
  }

  public FindVanNumber() {
    this.http.post("WH", "SNWR030_Get_Van_Number", this.params).subscribe(
      (res) => {
        if (res.Table[0]) {
          const van_no = res.Table[0].VAN_NO;
          this.global.GoToReport("PMS_RPT_WMS_SNWR030_VAN", {
            P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
            P_VAN_NO_FROM: van_no,
            P_VAN_NO_TO: van_no,
          });
        } else {
          this.modal.error("Unavailable Van Number.", this.viewRef);
        }
      },
      (err) => {
        this.modal.warning(
          "Found an error, please contract support team",
          this.viewRef
        );
      }
    );
  }

  public Print() {
    this.global.GoToReport("PMS_RPT_WMS_SNWR030_VAN", {
      P_COMPANY_CODE: this.global.GetTokenDetail("ssComCode"),
      P_VAN_NO_FROM: this.vanNoFrom,
      P_VAN_NO_TO: this.vanNoTo,
    });
  }

}
