import { Component, OnInit, ViewChild, ViewContainerRef } from '@angular/core';
import { HttpService } from 'src/app/core/http.service';
import { Router } from '@angular/router';
import { Globals } from 'src/app/core/global';
import { ModalService } from 'src/app/core/modal.service';

@Component({
    selector: 'SNWR032',
    templateUrl: 'SNWR032.component.html',
    styleUrls: ['SNWR032.component.css']
})

export class SNWR032Component implements OnInit {
    constructor(
        private http: HttpService,
        private router: Router,
        private global: Globals,
        private modal: ModalService,
        private viewref: ViewContainerRef
  ) { }

  @ViewChild("dialogAction", { static: false }) dialogAction: any;

  public paramSearch: any = {PART_NO: ""};
  public formData: any = {};
  public tableDataBabyCardLevel: Array<any> = [];


  public lookupPartNoConfig: any = {
    ColName: "LOOKUP_PART_NAME_EN",
    PlaceholderText: "Select Part no",
    SourceController: "WH",
    SourceMethodName: "SNWR032_GetPartNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true
  }

  public SetSelectedPartNo(val) {
    this.paramSearch.PART_NO = val.PART_NO;
    this.paramSearch.PART_NAME_TH = val.PART_NAME_TH;
    this.paramSearch.PART_NAME_EN = val.PART_NAME_EN;
    this.SearchBabyCardLevel();
  }

  public SetSelectedPartNoDialog(val) {
    if(val){
      this.formData.PART_NO = val.PART_NO;
      this.formData.PART_NAME_TH = val.PART_NAME_TH;
      this.formData.PART_NAME_EN = val.PART_NAME_EN;
    }else{
      this.formData.PART_NO = null;
      this.formData.PART_NAME_TH = null;
      this.formData.PART_NAME_EN = null;
    }
  }

  ngOnInit() {
    this.SearchBabyCardLevel();
  }

  public SearchBabyCardLevel() {
    this.http.post('WH', 'SNWR032_GetBabyCardLevelMaster', this.paramSearch)
    .subscribe(res => {
      if(res.result == "Done"){
        this.tableDataBabyCardLevel = res.set.Table;
      }
      else{
        this.modal.error(res.message, this.viewref)
      }
    },(err) => {
      console.log(err, 'err')
      this.modal.warning("Found an error, please contract support team", this.viewref);
  });
  }
  
  public Add() {
    this.formData = {}
    this.formData.TYPE = "ADD";
    this.dialogAction.height = "400px" 
    this.dialogAction.title = "Add Baby Card Level Master" 
    this.dialogAction.OpenDialog();
  }

  public Edit(data) {
    this.formData = { ...data }
    this.formData.TYPE = "EDIT";
    this.dialogAction.height = "450px" 
    this.dialogAction.title = "Edit Baby Card Level Master" 
    this.dialogAction.OpenDialog();
  }

  public SaveAdd() {
    console.log(this.formData);
    if (IsNullOrEmpty(this.formData.PART_NO)) {
      this.modal.warning("Please input fill part no.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_1)) {
      this.modal.warning("Please input validate value in level 1 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_2)) {
      this.modal.warning("Please input validate value in level 2 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_3)) {
      this.modal.warning("Please input validate value in level 3 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_4)) {
      this.modal.warning("Please input validate value in level 4 number.", this.viewref);
    } else {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmAdd);
    }
  }

  public ConfirmAdd = () => {
    this.http.post('WH', 'SNWR032_InsertBabyCardLevelMaster', this.formData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.SearchBabyCardLevel();
          this.dialogAction.Close();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
        } else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        } else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      })
  }

  public CancelAdd() {
    this.dialogAction.Close();
  }

  public SaveEdit() {
    console.log(this.formData);
    if (IsNullOrEmpty(this.formData.PART_NO)) {
      this.modal.warning("Please input fill part no.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_1)) {
      this.modal.warning("Please input validate value in level 1 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_2)) {
      this.modal.warning("Please input validate value in level 2 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_3)) {
      this.modal.warning("Please input validate value in level 3 number.", this.viewref);
    } else if (isNegativeNumber(this.formData.LEVEL_4)) {
      this.modal.warning("Please input validate value in level 4 number.", this.viewref);
    } else {
      this.modal.confirm("Do you want to save change you have made?", this.viewref, this.ConfirmEdit);
    }    
  }

  public ConfirmEdit = () => {
    this.http.post('WH', 'SNWR032_UpdateBabyCardLevelMaster', this.formData)
      .subscribe(res => {
        if (res.Table[0].STATUS == 'Done') {
          this.SearchBabyCardLevel();
          this.dialogAction.Close();
          this.modal.info('Transaction complete: applied and saved.', this.viewref);
        } else if (res.Table[0].STATUS == 'Duplicate') {
          this.modal.warning('No change to save. Found duplicate record.', this.viewref);
        } else {
          this.modal.error('No change to save. Something went wrong.', this.viewref);
        }
      }, (err) => {
        console.log(err, 'err')
        this.modal.warning("Found an error, please contract support team", this.viewref);
      });
  }

  public CancleEdit() {
    this.dialogAction.Close();
  }
}

function IsNullOrEmpty(value: any): boolean {
  if (value == '' || value == null || typeof value == 'undefined') {
    return true;
  } else {
    return false;
  }
}


function isNegativeNumber(value: any): boolean {
  if (typeof value == 'undefined') {
    return false;
  }
  if(isNaN(Number(value))){
    return true;
  }
  else{
    if (Number(value) == 0){
      return false;
    } 
    if(Number(value) < 0){
      return true;
    } else{
      return false;
    }
  }
}