import {
  Component,
  ElementRef,
  HostListener,
  OnInit,
  ViewChild,
  ViewContainerRef,
} from "@angular/core";
import { HttpService } from "src/app/core/http.service";
import { ModalService } from "src/app/core/modal.service";
import { Observable, Subject } from "rxjs";
import {
  PickToken,
  PickTokenExtractor,
} from "src/app/core/WHOutboundController/PickTokenExtractor";
import {
  InvoiceControl,
  InvoiceControlSetValue,
  CaseNo,
} from "src/app/core/WHOutboundController/InvoiceControl";
import { map } from "rxjs/operators";
import * as moment from "moment";
import { DialogTemplateComponent } from "src/app/components/shared/dialog-template/dialog-template.component";
import { Globals } from "src/app/core/global";

type ActionLog = {
  timestamp: moment.Moment;
  actionType: string;
  actionMessage: string;
};
@Component({
  selector: "SNWR033",
  templateUrl: "./SNWR033.component.html",
  styleUrls: ["../warehouse.component.css", "./SNWR033.component.css"],
  providers: [PickTokenExtractor],
})
export class SNWR033Component implements OnInit {
  constructor(
    private Http: HttpService,
    private modal: ModalService,
    private viewref: ViewContainerRef,
    private extractor: PickTokenExtractor,
    private global: Globals
  ) { }

  public qrString: string;
  public params: any = {};
  public tableData: any = [];
  public tableDataFiltered: any = [];
  public totalItems: number = 0;
  private prevPicking: PickToken;
  public pickingList: string[] = [];
  public tableDetail : any = []

  public showHistory: boolean = false;
  public actionLogs: ActionLog[] = [];
  private actionLogsSubject: Subject<ActionLog> = new Subject<ActionLog>();
  public lockQty: boolean = true;
  @ViewChild("inputQtyDialog") inputQtyDialog: DialogTemplateComponent;
  @ViewChild("inputCaseDialog") inputCaseDialog: DialogTemplateComponent;
  @ViewChild("saveButton", { read: ElementRef }) saveButton: ElementRef;
  @ViewChild("saveCaseTypeButton", { read: ElementRef })
  saveCaseTypeButton: ElementRef;
  @ViewChild("scannerInput", { read: ElementRef }) scannerInput: ElementRef;

  public initialControl: InvoiceControl = new InvoiceControl();
  private initialControlSubject: Subject<InvoiceControlSetValue> =
    new Subject<InvoiceControlSetValue>();

  public scannedQR: PickToken;

  public lookupPickListNoConfig: any = {
    ColName: "PICKING_LIST_NO",
    PlaceholderText: "Select Picking List No.",
    SourceController: "WH",
    SourceMethodName: "SNWR022_GetPickListNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    UpperCase: true,
  };

  public lookupOrderNoConfig: any = {
    ColName: "LOOKUP_ORDER_NO",
    PlaceholderText: "Select Order No",
    SourceController: "WH",
    SourceMethodName: "SNWR022_GetOrderNo",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      CUSTOMER_CODE: null,
      BRANCH_CODE: null,
    },
    UpperCase: true,
  };

  public lookupPickLotSeqConfig: any = {
    ColName: "LOOKUP_PICK_LOT_SEQ",
    PlaceholderText: "Select Pick Lot Seq",
    SourceController: "WH",
    SourceMethodName: "SNWR022_GetPickLotSeq",
    NoOfShowRecords: 50,
    MinLengthSearch: 0,
    SourceParams: {
      DATA: this.params,
    },
    UpperCase: true,
  };

  ngOnInit() {
    this.InitialProcess();

    this.initialControlSubject
      .pipe(map((control) => control))
      .subscribe((control) => {
        if (control) {
          this.initialControl.set(control);

          if (control.propName === "QR")
            // this.CheckFinishedForChangeInstruction() &&
            this.SetScannedQR(this.initialControl.QR);
          if (control.propName === "partNo")
            this.SetScannedPart(this.initialControl.partNo);
          if (control.propName === "baBy")
            this.SetScannedBaby(this.initialControl.baBy);
         
          this.qrString = undefined;
        }
      });

    this.actionLogsSubject.subscribe((action: ActionLog) => {
      this.actionLogs.push(action);
      this.actionLogs = this.actionLogs.slice();
    });
  }

  private InitialProcess() {
   
    this.totalItems = 0;
   
    this.lockQty = true;
    this.params = {};
    this.qrString = null;
    this.tableData = [];
    this.tableDataFiltered = [];
    this.pickingList = [];
    this.actionLogs = [];
    this.tableDetail = [];
    this.initialControl.reset();
   
    this.prevPicking = null;
  }


  public ToggleLockQty() {
    this.lockQty = !this.lockQty;
  }

  public Receive() {
    if (this.CheckFinishedForSaveInstruction()) {
      this.modal.confirm(
        "คุณต้องการบันทึกการทำงานใช่หรือไม่? ",
        this.viewref,
        this.ConfirmReceive
      );
    }
  }

  private ConfirmReceive = () => {

    this.Http.post("WH", "SNWR033_Receive", this.tableData).subscribe(
      (res) => {
        if (res.status === "Done") {
          this.modal.info(`บันทึกเสร็จสิ้น`, this.viewref, () => {
            this.scannerInput.nativeElement.focus();
          });
          this.InitialProcess();
        } else if (res.status === "Fail") {
          this.modal.error(res.massage, this.viewref);
        } else {
          this.modal.error(
            "พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",
            this.viewref
          );
        }
      },
      (err) => {
        this.modal.error(
          "พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",
          this.viewref
        );
      }
    );
  };

  public async Search() {
    this.tableData = [];
    this.tableDetail = [];
    if (this.params.CUSTOMER_CODE && this.params.BRANCH_CODE) {
      await this.Http.post("WH", "SNWR033_GetData", this.initialControl.QR)
        .toPromise()
        .then((res) => {

        
          // if(res.Table[0].INSTRUCTION_NO == "INVALID_PICKING"){
          //   this.modal.warning("หมายเลขการ์ดไม่ถูกต้องxx",this.viewref);
          //   this.initialControl.reset();
          //   return;
          // }
          this.tableData = res.Table;
          this.tableDetail.push(this.tableData[0])
          this.tableDetail = this.tableDetail.slice();

          const {
            CUSTOMER_CODE,
            BRANCH_CODE,
            CUSTOMER_NAME,
            BRANCH_NAME,
            CUSTOMER_ZONE,
          } = this.tableData[0];
          this.params = {
            CUSTOMER_CODE,
            BRANCH_CODE,
            CUSTOMER_NAME,
            BRANCH_NAME,
            CUSTOMER_ZONE,
          };
        });
    } else {
      this.modal.warning(
        "หมายเลขลูกค้าไม่ถูกต้อง",
        this.viewref
      );
    }
  }

  public OrderClassChange(event) {
    this.params.ORDER_CLASS = event.value;
  }

  private async SetScannedQR(pickToken: PickToken) {
  
      await this.Search();
      const foundPickingList = this.tableData.find(
        (item) => item.INSTRUCTION_NO === pickToken.INSTRUCTION_NO
      );
      if (foundPickingList) {
        this.prevPicking = pickToken;
        //this.FilterTableData();
        this.actionLogsSubject.next({
          timestamp: moment(),
          actionType: "Scan",
          actionMessage: `Picking List No ${this.initialControl.QR.PICKING_LIST_NO}, Instruction No ${this.initialControl.QR.INSTRUCTION_NO}`,
        });
      } else {
        //this.initialControl.QR = this.prevPicking;
        this.InitialProcess();
        this.modal.warning("ข้อมูลไม่ถูกต้อง", this.viewref);
      }
    this.scannerInput.nativeElement.focus();
  }


  private async SetScannedPart(partNo: string) {
    if (!this.initialControl.baBy) {
      this.modal.warning("กรุณาสแกนเบบี้การ์ด", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
      return;
    }
    let foundPart = this.tableData.find((item) =>
      item.BABYCARD_NO == this.initialControl.baBy && item.PART_NO == partNo
    ); //Find all matched part number

    if (foundPart) {
        let available = foundPart.BABY_QTY - foundPart.RECEIVE_QTY
        if(available > 0)
        {
          let index = this.tableData.findIndex(x => x.BABYCARD_NO == foundPart.BABYCARD_NO)
          
          if(foundPart.BABY_LEVEL == "LV3")
          {
            this.tableData[index].RECEIVE_QTY = foundPart.LEVEL3_QTY          
          }
          else
          {
            this.tableData[index].RECEIVE_QTY +=  foundPart.LEVEL1_QTY
          }
        }
        else
        {
          this.modal.error("รับเข้าเกินจำนวน ของเบบี้การ์ด", this.viewref, () => {
            this.scannerInput.nativeElement.focus();
          });
          return;
        }
      
    } else {
      //Case not found matched part number
      this.modal.error("เบอร์อะไหล่ไม่ถูกต้อง", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
    }
  }

  private async SetScannedBaby(baByNo: string) {
    if (!this.initialControl.QR.INSTRUCTION_NO) {
      this.modal.warning("กรุณาสแกนการ์ด", this.viewref, () => {
        this.scannerInput.nativeElement.focus();
      });
      return;
    }


    let foundBaby = this.tableData.find((item) =>
      item.BABYCARD_NO == baByNo
    ); //Find all matched part number

    if(foundBaby){
      this.modal.warning("หมายเลขเบบี้การ์ด " + foundBaby.BABYCARD_NO + " ถูกใช้งานแล้ว",this.viewref);
      return
    }

    let findBaby = this.tableData.find((item) =>
      item.BABYCARD_NO == null
    ); 

    if(!findBaby){
      this.modal.warning("เบบี้การ์ดครบจำนวนแล้ว",this.viewref);
      return
    }

    if(findBaby.BABY_LEVEL == 'LV1' && findBaby.TEMP_BABY != baByNo)
    {
      this.modal.warning("หมายเลขเบบี้การ์ดไม่ถูกต้อง",this.viewref);
      return
    }

    let index = this.tableData.findIndex(x => x.SEQ == findBaby.SEQ);
    this.tableData[index].BABYCARD_NO = baByNo;
    //this.tableData[index].RECEIVE_QTY = this.tableData[index].BABY_QTY;
  }


  private async GetBabyNumber(baByNo) {
    return await this.Http.get(
      "WH",
      "SNWR033_GetAvailableBaby",
      this.global.EscapeCharacter(baByNo)
    )
      .toPromise()
      .catch((err) => {
        this.modal.error(
          "พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",
          this.viewref,
          () => {
            this.scannerInput.nativeElement.focus();
          }
        );
      });
  }


  public async Enter() {
    try {
      const tokenDetail: PickToken = this.extractor.getToken(
        this.qrString.trim()
      );

      //Initial scan
      if (tokenDetail instanceof PickToken) {
        //Check if same customer
        if (
          this.params.CUSTOMER_CODE &&
          (tokenDetail.CUSTOMER_CODE !== this.params.CUSTOMER_CODE ||
            tokenDetail.BRANCH_CODE !== this.params.BRANCH_CODE)
        ) {
          this.modal.warning("เบอร์ลูกค้าไม่ตรงกับงานที่ทำอยู่", this.viewref, () => {
            this.scannerInput.nativeElement.focus();
          });
       
        } else if (this.tableData.length > 0 ){
            this.modal.warning("กรุณารับเข้างานให้เสร็จ", this.viewref);
            this.scannerInput.nativeElement.focus();
        } else {
          if (!this.params.CUSTOMER_CODE) {
            this.params = tokenDetail;
          }
          //Set QR code
          this.initialControlSubject.next({
            propName: "QR",
            value: tokenDetail,
          });
        }
      }
    } catch (ex) {
      if (ex === "Invalid QR") {

        const checkPart = this.tableData.find(
          (item) =>
            item.PART_NO === this.qrString &&
            item.INSTRUCTION_NO === this.initialControl.QR.INSTRUCTION_NO
        );
        if (checkPart) {
          this.initialControlSubject.next({
            propName: "partNo",
            value: this.qrString,
          });
        }

        
        let checkbaBy = null;
        if(!checkPart){
          let checkCust = this.qrString.split("-");
          let checkComplete = null;
          //let checkbaBy = null;

          if (checkCust[0] == this.params.CUSTOMER_CODE)
          {
            checkComplete = this.CheckCompleteBaby();
            checkbaBy = await this.GetBabyNumber(this.qrString).then((response) => {
              if (response.Result === "Done" && response.BABYCARD_NO === this.qrString) 
              {
                return response.BABYCARD_NO;
              }  
              else 
              {
                this.modal.warning("หมายเลขเบบี้การ์ดไม่สามารถใช้งานได้", this.viewref, () => {
                  this.scannerInput.nativeElement.focus();
                });
                return null;
              }
            });
          }
          else
          {
            this.modal.warning("ข้อมูลที่สแกนไม่ถูกต้อง", this.viewref, () => {
              this.scannerInput.nativeElement.focus();
            });
          }
        
          if (checkbaBy && !checkComplete && checkCust[0] == this.params.CUSTOMER_CODE) {
            this.initialControlSubject.next({
              propName: "baBy",
              value: checkbaBy,
            });
          }
        }else if (!checkPart && !checkbaBy) {
          this.modal.warning("ข้อมูลที่สแกนไม่ถูกต้อง", this.viewref, () => {
            this.scannerInput.nativeElement.focus();
          });
        }
        
      } else {
        this.modal.warning(
          "พบปัญหาการใช้งาน กรุณาแจ้งผู้ดูแลระบบ",
          this.viewref,
          () => {
            this.scannerInput.nativeElement.focus();
          }
        );
      }
    }
    this.qrString = null;
  }

  @HostListener("window:keydown", ["$event"])
  async onKeyDown(event: KeyboardEvent) {
    if (
      event.key.toUpperCase() === "ENTER" &&
      (<HTMLElement>event.target).id.toUpperCase() === "QRINPUT"
    ) {
      this.Enter();
    }
  }


  private CheckFinishedForSaveInstruction() {
    let foundUnfinishedInstruction = this.tableData.find(
      (item) => item.BABYCARD_NO == null
    );

    if (foundUnfinishedInstruction) {
      this.modal.warning("กรุณารับเข้างานให้เสร็จ", this.viewref);
      this.scannerInput.nativeElement.focus();
    }

    if(!foundUnfinishedInstruction){
			foundUnfinishedInstruction = this.tableData.find(
				(item) => item.BABY_QTY != item.RECEIVE_QTY
			  );

			if (foundUnfinishedInstruction) {
        this.modal.warning("กรุณารับเข้างานให้เสร็จ", this.viewref);
        this.scannerInput.nativeElement.focus();
			}
		}
    return !foundUnfinishedInstruction;
  }

  private CheckCompleteBaby(){
    let checkbaBy = this.tableData.some(x => x.BABYCARD_NO && x.BABYCARD_NO == this.initialControl.baBy && x.RECEIVE_QTY != x.BABY_QTY)

    if (checkbaBy) {
      this.modal.warning("กรุณาสแกนอะไหล่ให้ครบตามจำนวน", this.viewref);
      this.scannerInput.nativeElement.focus();
    }

    return checkbaBy;
  }

 
}
